%De Fiore and Uhlig, January 2015.
%coefficients.m computes the coefficients appearing in the log-linearized
%aggregation equations of the model presented in the paper "Corporate Debt Structure and the Financial Crisis" (JMCB 2015) by F. De Fiore and H. Uhlig. 

format long
tic;

%%% some definitions
eb =  s_bank/s_aggr; %threshold for eps1 above which firms choose banking; s_bank is calculated in choice.m
ec =  s_cmf/s_aggr; %threshold for eps1 above which firms choose CMFs; s_cmf is calculated in choice.m
om_eb_eps2_interp = interp1(rhsgrd2,omgrd2,s_bank*eps2_grd); %threshold for omega (below which the firm defaults) evaluated at eps1=eb
om_ec_eps2_interp = interp1(rhsgrd2,omgrd2,s_cmf*eps2_grd); %threshold for omega (below which the firm defaults) evaluated at eps1=eb
le=length(s_times_eps1_grd);
om_ed = interp1(rhsgrd2,omgrd2,s_times_eps1_grd.*eps2_cutoff);
om_ed_interp = interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,om_ed,0],s_ng); 
res_eb = (s_bank*eps2_grd).*f_func(om_eb_eps2_interp,sigma_eps3)*xi > 1; %dummy that takes the value of 1 when expected profits from continuing with banks (conditional on eps1=eb) are greater than one
[dummy_eb,ind_eb]=max(res_eb');
res_ec = (s_cmf*eps2_grd).*f_func(om_ec_eps2_interp,sigma_eps3)*xi > 1;
[dummy_ec,ind_ec]=max(res_ec');
ed_eb=eps2_grd(ind_eb); % this calculates e_d(eb,q,R);
ed_ec=eps2_grd(ind_ec); %this calculates e_d(ec,q,R);
res_fb1 = eps2_grd > ed_eb;
res_fb2 = eps2_grd > ed_ec;

%Recall that 
%prob_of_rectangle_original = (dens((s_ng/s_aggr+s_ng_stp/2),sigma_eps1)/s_aggr)*s_ng_stp;
%prob_of_rectangle = prob_of_rectangle_original/sum(prob_of_rectangle_original);
%eps2_weight_sig = eps2_stp*dens(eps2_grd+eps2_stp/2,sigma_eps2);
%prob_of_rect_eps2 = eps2_weight/sum(eps2_weight);
dens_sig1 = ( -1/sigma_eps1 - (1/sigma_eps1)*(log(s_ng/s_aggr+s_ng_stp/2)+.5*sigma_eps1^2) + (1/(sigma_eps1^3))*((log(s_ng/s_aggr+s_ng_stp/2) + .5*sigma_eps1^2).^2) );
dens_sig2 = ( -1/sigma_eps2 - (1/sigma_eps2)*(log(eps2_grd)+.5*sigma_eps2^2) + (1/(sigma_eps2^3))*((log(eps2_grd) + .5*sigma_eps2^2).^2) );

%%% Derivatives of the threshold function e_d(eps1,q,R,sigma3)
ed_eps1_interp = interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0, eps2_cutoff,0],s_ng);      %eps2_cutoff is computed in bank_fill_in_vec 
Ded_Deps1_interp = - ed_eps1_interp./(s_ng./s_aggr);
Ded_Deps1_eb_interp = Ded_Deps1_interp(j_bank);
Ded_Deps1_ec_interp = Ded_Deps1_interp(j_cmf);

Ded_Dq_interp = - (1/s_aggr)*ed_eps1_interp; %derivative of ed(eps1;q,R) wrt q when eps1=s_ng/s_aggr
Ded_Dq_eb_interp = Ded_Dq_interp(j_bank); %derivative of ed(eps1;q,R) wrt q when eps1=eb
Ded_Dq_ec_interp = Ded_Dq_interp(j_cmf);

om_eps1_ed_interp = interp1(rhsgrd2,omgrd2,s_ng.*ed_eps1_interp); % threshold omega for bank financed firms, evaluated at eps1=eps1_grd and eps2=ed(eps1_grd;q,R)
Ded_DR_interp = (1/Rnom)*ed_eps1_interp.*(1 / (1 - f_func(om_eps1_ed_interp,sigma_eps3)/(f_prime(om_eps1_ed_interp,sigma_eps3).*omega_tilde(om_eps1_ed_interp,sigma_eps3,mon)))); %derivative of ed(eps1;q,R) wrt R when eps1=eps1_grd
Ded_DR_eb_interp = Ded_DR_interp(j_bank);
Ded_DR_ec_interp = Ded_DR_interp(j_cmf);

Ded_Dsigma3_interp = ed_eps1_interp.*(f_second(om_eps1_ed_interp,sigma_eps3)./f_func(om_eps1_ed_interp,sigma_eps3)...
                                       - f_prime(om_eps1_ed_interp,sigma_eps3).*g_second(om_eps1_ed_interp,sigma_eps3,mon)...
                                         ./(f_func(om_eps1_ed_interp,sigma_eps3).*g_prime(om_eps1_ed_interp,sigma_eps3,mon)))./...
                     (1 - f_prime(om_eps1_ed_interp,sigma_eps3).*omega_tilde(om_eps1_ed_interp,sigma_eps3,mon)./f_func(om_eps1_ed_interp,sigma_eps3));                          

%%% Derivative of ed(eps1;q,R,sigma3) wrt R when eps1=eps1_grd
Ded_Dsigma3_eb_interp = Ded_Dsigma3_interp(j_bank);
Ded_Dsigma3_ec_interp = Ded_Dsigma3_interp(j_cmf);

Ded_Dvarsigma_interp = Rnom*Ded_DR_interp; %derivative of ed(eps1;q,R,sigma3) wrt varsigma when eps1=eps1_grd
Ded_Dvarsigma_eb_interp = Ded_Dvarsigma_interp(j_bank);
Ded_Dvarsigma_ec_interp = Ded_Dvarsigma_interp(j_cmf);

%%% Derivatives of the threshold function omega_b(eps1,eps2,q,R,tau,varsigma,sigma3).  
le2 = length(eps2_grd);
Dombank_Deps1 = - s_aggr*omega_tilde(om,sigma_eps3,mon)./(s_times_eps1_grd'*ones(1,le2));
Dombank_Deps1_eb_interp = - 1/eb*omega_tilde(om_eb_eps2_interp,sigma_eps3,mon); %derivative wrt eps1 at eps1=eb and eps2=eps2_grd
Dombank_Deps1_ec_interp = - 1/ec*omega_tilde(om_ec_eps2_interp,sigma_eps3,mon);

Dombank_Dq = - 1/s_aggr*omega_tilde(om,sigma_eps3,mon); %derivative of the omega_b(eps1,eps2,q,R,tau,varsigma,sigma3) wrt q, evaluated at eps1=eps1_grd and eps2 = eps2_grd
Dombank_Dq_eb = - 1/s_aggr*omega_tilde(om_eb_eps2_interp,sigma_eps3,mon); %derivative of the omega_b(eps1,eps2,q,R,tau,varsigma,sigma2,sigma3) wrt q, evaluated at eps1=eb  and eps2 = eps2_grd
Dombank_Dq_ec = - 1/s_aggr*omega_tilde(om_ec_eps2_interp,sigma_eps3,mon); %derivative of the threshold function omega_b(eps1,eps2,q,R,tau,varsigma,sigma2,sigma3) wrt q, evaluated at eps1=ec  and eps2 = eps2_grd

Dombank_DR = 1/Rnom*omega_tilde(om,sigma_eps3,mon); %derivative of the omega_b(eps1,eps2,q,R,tau,varsigma,sigma3) wrt R, evaluated at eps1=eps1_grd and eps2 = eps2_grd
Dombank_DR_eb_interp = 1/Rnom*omega_tilde(om_eb_eps2_interp,sigma_eps3,mon);
Dombank_DR_ec_interp = 1/Rnom*omega_tilde(om_ec_eps2_interp,sigma_eps3,mon);

Dombank_Dvarsigma = omega_tilde(om,sigma_eps3,mon); %derivative of the omega_b(eps1,eps2,q,R,tau,varsigma,sigma3) wrt varsigma, evaluated at eps1=eps1_grd and eps2 = eps2_grd
Dombank_Dvarsigma_eb_interp = omega_tilde(om_eb_eps2_interp,sigma_eps3,mon);
Dombank_Dvarsigma_ec_interp = omega_tilde(om_ec_eps2_interp,sigma_eps3,mon);

Dombank_Dsigma3 = - g_second(om,sigma_eps3,mon)./g_prime(om,sigma_eps3,mon); %derivative of the omega_b(eps1,eps2,q,R,tau,varsigma,sigma3) wrt sigma3, evaluated at eps1=eps1_grd and eps2 = eps2_grd
Dombank_Dsigma3_eb_interp = - g_second(om_eb_eps2_interp,sigma_eps3,mon)./g_prime(om_eb_eps2_interp,sigma_eps3,mon);
Dombank_Dsigma3_ec_interp = - g_second(om_ec_eps2_interp,sigma_eps3,mon)./g_prime(om_ec_eps2_interp,sigma_eps3,mon);

%%% Derivatives of the function omega_cmf(eps1,q,R,sigma2,sigma3)
om_cmf_interp = interp1(rhsgrd2,omgrd2,s_ng);
Domcmf_Dq = -1/s_aggr* omega_tilde(om_cmf,sigma_cmf,mon);
Domcmf_Dq_interp = -1/s_aggr*omega_tilde(om_cmf_interp,sigma_cmf,mon);
Domcmf_DR = 1/Rnom* omega_tilde(om_cmf,sigma_cmf,mon);
Domcmf_DR_interp = 1/Rnom*omega_tilde(om_cmf_interp,sigma_cmf,mon);
Domcmf_Dsigma2_interp = -(sigma_eps2/sigma_cmf)*g_second(om_cmf_interp,sigma_cmf,mon)./g_prime(om_cmf_interp,sigma_cmf,mon);
Domcmf_Dsigma3_interp = -(sigma_eps3/sigma_cmf)*g_second(om_cmf_interp,sigma_cmf,mon)./g_prime(om_cmf_interp,sigma_cmf,mon);


%%% Fb(eps1,q,R, varsigma) evaluated at eb and ec, needed for the computation of phif_q and phif_R
%Fb_interp is computed in steadystate_quarterly as interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0, profits_bank,0],s_ng);
Fb_ec_interp = Fb_interp(j_cmf);  

%%% Derivative of Fb(eps1,q,R,tau,varsigma,sigma2,sigma3) wrt eps1
%Remember that om_eb_eps2_interp = interp1(rhsgrd2,omgrd2,s_bank*eps2_grd) and om_eb_edeb = interp1(rhsgrd2,omgrd2,s_bank*ed_eb);
Fb_1_mat1_interp = - xi*s_ng.*Ded_Deps1_interp.*ed_eps1_interp.*f_func(om_eps1_ed_interp,sigma_eps3).*dens(ed_eps1_interp,sigma_eps2);
% res_o is defined in bank_fill_in_vec as res_o = ((s_times_eps1_grd'*eps2_grd).*f_func(om,sigma_eps3)-chi)*xi > 1;
Fb_1_mat2a = (s_aggr*(ones(le,1)*eps2_grd).*f_func(om,sigma_eps3) + (s_times_eps1_grd'*eps2_grd).*f_prime(om,sigma_eps3).*Dombank_Deps1).*(ones(le,1)*dens(eps2_grd+eps2_stp/2,sigma_eps2));
Fb_1_mat2 = sum( (xi*res_o.*Fb_1_mat2a)')*eps2_quad_stp ;
Fb_1_mat2_interp = interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,Fb_1_mat2,0],s_ng);
Fb_1_mat3_interp = (1+chi*xi)*dens(ed_eps1_interp,sigma_eps2).*Ded_Deps1_interp;
Fb_1_interp = (1-tau)*( Fb_1_mat1_interp + Fb_1_mat2_interp + Fb_1_mat3_interp );
Fb_1_eb_interp = Fb_1_interp(j_bank);
Fb_1_ec_interp = Fb_1_interp(j_cmf);    

%%% Derivative of Fb(eps1,q,R,tau, varsigma,sigma2,sigma3) wrt q, to be used in phif_q and phif_R
Fb_2_mat1_interp = - xi*s_ng.*DUMMYD.*Ded_Dq_interp.*ed_eps1_interp.*f_func(om_eps1_ed_interp,sigma_eps3).*dens(ed_eps1_interp,sigma_eps2);
Fb_2_mat2a = (s_times_eps1_grd'*eps2_grd).*(1/s_aggr*f_func(om,sigma_eps3) + f_prime(om,sigma_eps3).*Dombank_Dq);
Fb_2_mat2 = sum( (xi*res_o.*Fb_2_mat2a.*(ones(le,1)*prob_of_rect_eps2))' );
Fb_2_mat2_interp = interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,Fb_2_mat2,0],s_ng);
Fb_2_mat3_interp = (1+chi*xi)*dens(ed_eps1_interp,sigma_eps2).*DUMMYD.*Ded_Dq_interp;
Fb_2_interp = (1-tau)*( Fb_2_mat1_interp + Fb_2_mat2_interp + Fb_2_mat3_interp );
Fb_2_eb_interp = Fb_2_interp(j_bank);
Fb_2_ec_interp = Fb_2_interp(j_cmf);

%%% Derivative of Fb(eps1,q,R,tau, varsigma,sigma2,sigma3) wrt R, to be used in phif_q and phif_R
Fb_3_mat1_interp = - xi*s_ng.*DUMMYD.*Ded_DR_interp.*ed_eps1_interp.*f_func(om_eps1_ed_interp,sigma_eps3).*dens(ed_eps1_interp,sigma_eps2);
Fb_3_mat2a = (s_times_eps1_grd'*eps2_grd).*f_prime(om,sigma_eps3).*Dombank_DR;
Fb_3_mat2 = sum( (xi*res_o.*Fb_3_mat2a.*(ones(le,1)*prob_of_rect_eps2))' );
Fb_3_mat2_interp = interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,Fb_3_mat2,0],s_ng);
Fb_3_mat3_interp = (1+chi*xi)*dens(ed_eps1_interp,sigma_eps2).*DUMMYD.*Ded_DR_interp;
Fb_3_interp = (1-tau)*( Fb_3_mat1_interp + Fb_3_mat2_interp + Fb_3_mat3_interp );
Fb_3_eb_interp = Fb_3_interp(j_bank);
Fb_3_ec_interp = Fb_3_interp(j_cmf);

%%% Derivative of Fb(eps1,q,R,tau,varsigma,sigma2,sigma3) wrt tau, to be used in phif_tau and phif_tau
Fb_4_interp = - (1/(1-tau))*Fb_interp;
Fb_4_eb_interp = Fb_4_interp(j_bank);
Fb_4_ec_interp = Fb_4_interp(j_cmf);

%%% Derivative of Fb(eps1,q,R,tau, varsigma,sigma2,sigma3) wrt varsigma, to be used in phif_q and phif_R
Fb_5_mat1_interp = - xi*s_ng.*DUMMYD.*Ded_Dvarsigma_interp.*ed_eps1_interp.*f_func(om_eps1_ed_interp,sigma_eps3).*dens(ed_eps1_interp,sigma_eps2);
Fb_5_mat2a = (s_times_eps1_grd'*eps2_grd).*f_prime(om,sigma_eps3).*Dombank_Dvarsigma;
Fb_5_mat2 = sum( (xi*res_o.*Fb_5_mat2a.*(ones(le,1)*prob_of_rect_eps2))' );
Fb_5_mat2_interp = interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,Fb_5_mat2,0],s_ng);
Fb_5_mat3_interp = (1+chi*xi)*dens(ed_eps1_interp,sigma_eps2).*DUMMYD.*Ded_Dvarsigma_interp;
Fb_5_interp = (1-tau)*( Fb_5_mat1_interp + Fb_5_mat2_interp + Fb_5_mat3_interp );
Fb_5_eb_interp = Fb_5_interp(j_bank);
Fb_5_ec_interp = Fb_5_interp(j_cmf);

%%% Derivative of Fb(eps1,q,R,tau,varsigma,sigma2,sigma3) wrt sigma2, to be used in phif_sigma2
Fb_sigma2_mat1a = (s_times_eps1_grd'*eps2_grd).*f_func(om,sigma_eps3);
Fb_sigma2_mat1 = sum( (xi*res_o.*Fb_sigma2_mat1a.*(ones(le,1)*(dens_sig2.*prob_of_rect_eps2)))' );
Fb_sigma2_mat1_interp = interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,Fb_sigma2_mat1,0],s_ng);
Fb_sigma2_mat2_interp = cdf_sig(ed_eps1_interp,sigma_eps2);
Fb_sigma2_interp = (1-tau)*( Fb_sigma2_mat1_interp + Fb_sigma2_mat2_interp );
Fb_sigma2_ec_interp = Fb_sigma2_interp(j_cmf);
Fb_sigma2_eb_interp = Fb_sigma2_interp(j_bank);

%%% Derivative of Fb(eps1,q,R,tau,varsigma,sigma2,sigma3) wrt sigma3, to be used in phif_sigma3
Fb_sigma3_mat1_interp = - xi*s_ng.*DUMMYD.*Ded_Dsigma3_interp.*ed_eps1_interp.*f_func(om_eps1_ed_interp,sigma_eps3).*dens(ed_eps1_interp,sigma_eps2);
Fb_sigma3_mat2a = (s_times_eps1_grd'*eps2_grd).*(f_prime(om,sigma_eps3).*Dombank_Dsigma3 + f_second(om,sigma_eps3));
Fb_sigma3_mat2 = sum( (xi*res_o.*Fb_sigma3_mat2a.*(ones(le,1)*prob_of_rect_eps2))' );
Fb_sigma3_mat2_interp = interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,Fb_sigma3_mat2,0],s_ng);
Fb_sigma3_mat3_interp = (1+chi*xi)*dens(ed_eps1_interp,sigma_eps2).*DUMMYD.*Ded_Dsigma3_interp;
Fb_sigma3_interp = (1-tau)*( Fb_sigma3_mat1_interp + Fb_sigma3_mat2_interp + Fb_sigma3_mat3_interp );
Fb_sigma3_eb_interp = Fb_sigma3_interp(j_bank);
Fb_sigma3_ec_interp = Fb_sigma3_interp(j_cmf);   

%%% Function F_c(eps1,q,R,sigma2,sigma3) and derivatives evaluated at eps1=ec 
%Recall that om_cmf_interp= interp1(rhsgrd2,omgrd2,s_ng), 
%Fc_interp is computed in steadystate_quarterly as xi*s_ng.*f_func(om_cmf_interp,sigma_cmf)-chi*xi;
Fc_ec_interp = Fc_interp(j_cmf);
omcmf_ec = om_cmf_interp(j_cmf);
Fc_1_interp = ((Fc_interp + xi*chi)./(1/s_aggr*s_ng)).*(1 - (f_prime(om_cmf_interp,sigma_cmf).*omega_tilde(om_cmf_interp,sigma_cmf,mon))/f_func(om_cmf_interp,sigma_cmf));
Fc_1_ec_interp = Fc_1_interp(j_cmf);
Fc_2_interp =   1/s_aggr*(Fc_interp + chi*xi).*(1 - (f_prime(om_cmf_interp,sigma_cmf).*omega_tilde(om_cmf_interp,sigma_cmf,mon))./f_func(om_cmf_interp,sigma_cmf));
Fc_2_ec_interp = Fc_2_interp(j_cmf);
Fc_3_interp= 1/Rnom*(Fc_interp + chi*xi).* (f_prime(om_cmf_interp,sigma_cmf).*omega_tilde(om_cmf_interp,sigma_cmf,mon) )./f_func(om_cmf_interp,sigma_cmf);
Fc_3_ec_interp = Fc_3_interp(j_cmf);

Fc_sigma2_interp =   (sigma_eps2/sigma_cmf)*s_aggr*ec*xi*( f_second(om_cmf_interp,sigma_cmf) -  (f_prime(om_cmf_interp,sigma_cmf).*g_second (om_cmf_interp,sigma_cmf,mon))./g_prime(om_cmf_interp,sigma_cmf,mon));
Fc_sigma2_eb_interp = Fc_sigma2_interp(j_bank);
Fc_sigma2_ec_interp = Fc_sigma2_interp(j_cmf);

Fc_sigma3_interp = (sigma_eps3/sigma_cmf)*s_aggr*ec*xi*( f_second(om_cmf_interp,sigma_cmf) -  (f_prime(om_cmf_interp,sigma_cmf).*g_second (om_cmf_interp,sigma_cmf,mon))./g_prime(om_cmf_interp,sigma_cmf,mon));
Fc_sigma3_eb_interp = Fc_sigma3_interp(j_bank);
Fc_sigma3_ec_interp = Fc_sigma3_interp(j_cmf);

%%% Derivatives of threshold ec
Dec_Dq_interp = - ( Fb_2_ec_interp - Fc_2_ec_interp )/( Fb_1_ec_interp - Fc_1_ec_interp );
Dec_DR_interp = - ( Fb_3_ec_interp - Fc_3_ec_interp )/( Fb_1_ec_interp - Fc_1_ec_interp );
Dec_Dtau_interp = - Fb_4_ec_interp /( Fb_1_ec_interp - Fc_1_ec_interp );
Dec_Dvarsigma_interp = - ( Fb_5_ec_interp )/( Fb_1_ec_interp - Fc_1_ec_interp );
Dec_Dsigma2_interp = - ( Fb_sigma2_ec_interp - Fc_sigma2_ec_interp )/( Fb_1_ec_interp - Fc_1_ec_interp );
Dec_Dsigma3_interp = - ( Fb_sigma3_ec_interp - Fc_sigma3_ec_interp )/( Fb_1_ec_interp - Fc_1_ec_interp );

%%% Derivatives of threshold eb
Deb_Dq_interp   = - Fb_2_eb_interp / Fb_1_eb_interp ;
Deb_DR_interp   = - Fb_3_eb_interp / Fb_1_eb_interp ;
Deb_Dtau_interp = - Fb_4_eb_interp / Fb_1_eb_interp ;
Deb_Dvarsigma_interp   = - Fb_5_eb_interp / Fb_1_eb_interp ;
Deb_Dsigma2_interp   = - Fb_sigma2_eb_interp / Fb_1_eb_interp ;
Deb_Dsigma3_interp   = - Fb_sigma3_eb_interp / Fb_1_eb_interp ;


%%% Computing the derivatives of phiy 
%remember that res_fb1 = eps2_grd > ed_eb and  res_fb2 = eps2_grd > ed_ec;
phiy_q = -DUMMYC*Dec_Dq_interp*ec*dens(ec,sigma_eps1)...
         + (1-tau)*( DUMMYC*Dec_Dq_interp*ec*dens(ec,sigma_eps1)*sum(  meandens(eps2_grd).*res_fb2 )*eps2_stp... 
                   - DUMMYB*Deb_Dq_interp*eb*dens(eb,sigma_eps1)*sum(  meandens(eps2_grd).*res_fb1 )*eps2_stp...
                   - (sum((1/s_aggr)*(1-undef_bank).*res.*(1-res2).*DUMMYD.*Ded_Dq_interp.*s_ng.*...
                           ed_eps1_interp.*dens(ed_eps1_interp,sigma_eps2).*prob_of_rectangle)));
phiy_R = -DUMMYC*Dec_DR_interp*ec*dens(ec,sigma_eps1)...
         + (1-tau)*( DUMMYC*Dec_DR_interp*ec*dens(ec,sigma_eps1)*sum(  meandens(eps2_grd).*res_fb2 )*eps2_stp... 
                   - DUMMYB*Deb_DR_interp*eb*dens(eb,sigma_eps1)*sum(  meandens(eps2_grd).*res_fb1 )*eps2_stp...
                   - sum((1/s_aggr)*(1-undef_bank).*res.*(1-res2).*s_ng.*DUMMYD.*Ded_DR_interp.*...
                         ed_eps1_interp.*dens(ed_eps1_interp,sigma_eps2).*prob_of_rectangle));
phiy_tau = -DUMMYC*Dec_Dtau_interp*ec*dens(ec,sigma_eps1)...
         + (1-tau)*( DUMMYC*Dec_Dtau_interp*ec*dens(ec,sigma_eps1)*sum(  meandens(eps2_grd).*res_fb2 )*eps2_stp... 
                   - DUMMYB*Deb_Dtau_interp*eb*dens(eb,sigma_eps1)*sum(  meandens(eps2_grd).*res_fb1 )*eps2_stp)...
                   - sum((1-undef_bank).*res.*(1-res2).*y_mat_interp.*prob_of_rectangle);
phiy_varsigma = -DUMMYC*Dec_Dvarsigma_interp*ec*dens(ec,sigma_eps1)...
         + (1-tau)*( DUMMYC*Dec_Dvarsigma_interp*ec*dens(ec,sigma_eps1)*sum(  meandens(eps2_grd).*res_fb2 )*eps2_stp... 
                   - DUMMYB*Deb_Dvarsigma_interp*eb*dens(eb,sigma_eps1)*sum(  meandens(eps2_grd).*res_fb1 )*eps2_stp...
                   - sum((1/s_aggr)*(1-undef_bank).*res.*(1-res2).*s_ng.*DUMMYD.*Ded_Dvarsigma_interp.*...
                         ed_eps1_interp.*dens(ed_eps1_interp,sigma_eps2).*prob_of_rectangle));
    
%remember that
%y_mat = sum( ((1/s_aggr)*(s_times_eps1_grd'*eps2_grd).*res_o.*(ones(le,1)*prob_of_rect_eps2))');
%y_mat_interp = interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,y_mat,0],s_ng);
%phiy = (1-tau)*sum((1-undef_bank).*res.*(1-res2).*y_mat_interp.*prob_of_rectangle)...
%      + sum((1/s_aggr)*(1-undef_cmf).*res2.*res3.*s_ng.*prob_of_rectangle); 
phiy_sigma1 = (1-tau)*sum((1-undef_bank).*res.*(1-res2).*y_mat_interp.*dens_sig1.*prob_of_rectangle)...
      + sum((1/s_aggr)*(1-undef_cmf).*res2.*res3.*s_ng.*dens_sig1.*prob_of_rectangle); 
          
phiy_sigma2_a = -DUMMYC*Dec_Dsigma2_interp*ec*dens(ec,sigma_eps1)...
                + (1-tau)*( DUMMYC*Dec_Dsigma2_interp*ec*dens(ec,sigma_eps1)*sum(  meandens(eps2_grd).*res_fb2 )*eps2_stp... 
                - DUMMYB*Deb_Dsigma2_interp*eb*dens(eb,sigma_eps1)*sum(  meandens(eps2_grd).*res_fb1 )*eps2_stp); %remember that ded_dsigma2=0;
phiy_sigma2_bit1 = sum((res_o.*(s_times_eps1_grd'*eps2_grd).*(ones(le,1)*(dens_sig2.*prob_of_rect_eps2)))');
phiy_sigma2_bit1_interp = interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,phiy_sigma2_bit1,0],s_ng);
phiy_sigma2_b = (1-tau)*( (sum((1/s_aggr)*(1-undef_bank).*res.*(1-res2).*phiy_sigma2_bit1_interp.*prob_of_rectangle)));
phiy_sigma2 = phiy_sigma2_a +phiy_sigma2_b;

phiy_sigma3 = -DUMMYC*Dec_Dsigma3_interp*ec*dens(ec,sigma_eps1)...
              + (1-tau)*( DUMMYC*Dec_Dsigma3_interp*ec*dens(ec,sigma_eps1)*sum(  meandens(eps2_grd).*res_fb2 )*eps2_stp... 
              - DUMMYB*Deb_Dsigma3_interp*eb*dens(eb,sigma_eps1)*sum(  meandens(eps2_grd).*res_fb1 )*eps2_stp...
              - (sum((1/s_aggr)*(1-undef_bank).*res.*(1-res2).*DUMMYD.*Ded_Dsigma3_interp.*s_ng.*...
                ed_eps1_interp.*dens(ed_eps1_interp,sigma_eps2).*prob_of_rectangle)));
                           
%%% Computing derivatives of phimb
%Recall that  expected_bank_monitoring = sum(frac_bank_monitored_withchoice.*prob_of_rectangle); %computed in steatystate.m
phimb = expected_bank_monitoring;
om_ec_eps2_interp = interp1(rhsgrd2,omgrd2,s_cmf*eps2_grd); %threshold for omega (below which the firm defaults) evaluated at eps1=ec
%recall that res_ec = (s_cmf*eps2_grd).*f_func(om_ec_eps2_interp,sigma_eps3)*xi > 1; similarly for res_eb
om_frac_ec = normcdf( log( om_ec_eps2_interp + sigma_eps3^2/2)/sigma_eps3 );
frac_bank_monitored_ec = sum((res_ec.*om_frac_ec.*eps2_weight)');
om_eb_eps2_interp = interp1(rhsgrd2,omgrd2,s_bank*eps2_grd); 
om_frac_eb = normcdf( log( om_eb_eps2_interp + sigma_eps3^2/2)/sigma_eps3 );
frac_bank_monitored_eb = sum((res_eb.*om_frac_eb.*eps2_weight)');
om_ed_eps1 =interp1(rhsgrd2,omgrd2,s_times_eps1_grd.*eps2_cutoff);
om_frac_ed_eps1 = normcdf( log( om_ed_eps1 + sigma_eps3^2/2)/sigma_eps3 );
phimb_bit1 = om_frac_ed_eps1.*dens(eps2_cutoff,sigma_eps2);
phimb_q_bit1_interp = - DUMMYD.*Ded_Dq_interp.*interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,phimb_bit1,0],s_ng);
phimb_q_bit2 = sum((res_o.*Dombank_Dq.*dens(om,sigma_eps3).*(ones(le,1)*eps2_weight))');
phimb_q_bit2_interp = interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,phimb_q_bit2,0],s_ng);
phimb_q =  DUMMYC*Dec_Dq_interp*dens(ec,sigma_eps1)*frac_bank_monitored_ec...
         - DUMMYB*Deb_Dq_interp*dens(eb,sigma_eps1)*frac_bank_monitored_eb...
         + sum((1-undef_cmf).*res.*(1-res2).*(phimb_q_bit1_interp+phimb_q_bit2_interp).*prob_of_rectangle);

phimb_R_bit1_interp = - DUMMYD.*Ded_DR_interp.*interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,phimb_bit1,0],s_ng);
phimb_R_bit2 = sum((res_o.*Dombank_DR.*dens(om,sigma_eps3).*(ones(le,1)*eps2_weight))');
phimb_R_bit2_interp = interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,phimb_R_bit2,0],s_ng);
phimb_R =  DUMMYC*Dec_DR_interp*dens(ec,sigma_eps1)*frac_bank_monitored_ec...
         - DUMMYB*Deb_DR_interp*dens(eb,sigma_eps1)*frac_bank_monitored_eb...
         + sum((1-undef_cmf).*res.*(1-res2).*(phimb_R_bit1_interp+phimb_R_bit2_interp).*prob_of_rectangle);

phimb_tau =  DUMMYC*Dec_Dtau_interp*dens(ec,sigma_eps1)*frac_bank_monitored_ec...
         - DUMMYB*Deb_Dtau_interp*dens(eb,sigma_eps1)*frac_bank_monitored_eb;


phimb_varsigma_bit1_interp = - DUMMYD.*Ded_Dvarsigma_interp.*interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,phimb_bit1,0],s_ng);
phimb_varsigma_bit2 = sum((res_o.*Dombank_Dvarsigma.*dens(om,sigma_eps3).*(ones(le,1)*eps2_weight))');
phimb_varsigma_bit2_interp = interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,phimb_varsigma_bit2,0],s_ng);
phimb_varsigma =  DUMMYC*Dec_Dvarsigma_interp*dens(ec,sigma_eps1)*frac_bank_monitored_ec...
         - DUMMYB*Deb_Dvarsigma_interp*dens(eb,sigma_eps1)*frac_bank_monitored_eb...
         + sum((1-undef_bank).*res.*(1-res2).*(phimb_varsigma_bit1_interp+phimb_varsigma_bit2_interp).*prob_of_rectangle);
 
%recall that
%frac_bank_monitored = sum((res_o.*om_frac.*(ones(le,1)*eps2_weight))');
%frac_bank_monitored_withchoice =(1-undef_bank).*res.*(1-res2).*interp1([s_bot-1,sgrd_bank,s_top+1],[0,frac_bank_monitored,0],s_ng)
phimb_sigma1 = sum(frac_bank_monitored_withchoice.*dens_sig1.*prob_of_rectangle);

phimb_sigma2_bit1 = sum((res_o.*om_frac.*(ones(le,1)*(dens_sig2.*prob_of_rect_eps2)))');
phimb_sigma2_bit1_interp = interp1([s_bot-1,sgrd_bank,s_top+1],[0,phimb_sigma2_bit1,0],s_ng);
phimb_sigma2 = DUMMYC*Dec_Dsigma2_interp*dens(ec,sigma_eps1)*frac_bank_monitored_ec...
             - DUMMYB*Deb_Dsigma2_interp*dens(eb,sigma_eps1)*frac_bank_monitored_eb...
             + sum((1-undef_bank).*res.*(1-res2).*phimb_sigma2_bit1_interp.*prob_of_rectangle);

om_frac_ed_interp = normcdf( log( om_ed_interp + sigma_eps3^2/2)/sigma_eps3 );
eps2_cutoff_interp = interp1([s_bot-1,sgrd_bank,s_top+1],[0,eps2_cutoff,0],s_ng);
phimb_sigma3_bit1_interp = - DUMMYD.*Ded_Dsigma3_interp.*om_frac_ed_interp.*dens(eps2_cutoff_interp,sigma_eps2);
phimb_sigma3_bit2 = sum((res_o.*(Dombank_Dsigma3.*dens(om,sigma_eps3) + cdf_sig(om,sigma_eps3)).*(ones(le,1)*eps2_weight))');
phimb_sigma3_bit2_interp = interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,phimb_sigma3_bit2,0],s_ng);
phimb_sigma3 = DUMMYC*Dec_Dsigma3_interp*dens(ec,sigma_eps1)*frac_bank_monitored_ec...
             - DUMMYB*Deb_Dsigma3_interp*dens(eb,sigma_eps1)*frac_bank_monitored_eb...
             + sum((1-undef_bank).*res.*(1-res2).*(phimb_sigma3_bit1_interp+phimb_sigma3_bit2_interp).*prob_of_rectangle);

%%% Computing derivatives of phimc
phimc = expected_cmf_monitoring;
om_cmf_ec_interp = om_cmf_interp(j_cmf);
frac_mon_ec_interp = normcdf(log(om_cmf_ec_interp+sigma_cmf^2/2)/sigma_cmf);

phimc_q =  sum((1-undef_cmf).*res2.*res3.*Domcmf_Dq.*dens(om_cmf,sigma_cmf).*prob_of_rectangle)...
         - DUMMYC*Dec_Dq_interp*dens(ec,sigma_eps1)*normcdf( log( om_cmf_ec_interp + sigma_cmf^2/2)/sigma_cmf );

phimc_R =  sum((1-undef_cmf).*res2.*res3.*Domcmf_DR.*dens(om_cmf,sigma_cmf).*prob_of_rectangle)...
         - DUMMYC*Dec_DR_interp*dens(ec,sigma_eps1)*normcdf( log( om_cmf_ec_interp + sigma_cmf^2/2)/sigma_cmf );

phimc_tau = - DUMMYC*Dec_Dtau_interp*dens(ec,sigma_eps1)*normcdf( log( om_cmf_ec_interp + sigma_cmf^2/2)/sigma_cmf );

phimc_varsigma =  - DUMMYC*Dec_Dvarsigma_interp*dens(ec,sigma_eps1)*normcdf( log( om_cmf_ec_interp + sigma_cmf^2/2)/sigma_cmf );

phimc_sigma1 =  sum(frac_cmf_monitored_withchoice.*dens_sig1.*prob_of_rectangle);

phimc_sigma2 = sum((1-undef_cmf).*res2.*res3.*(dens(om_cmf,sigma_eps1).*Domcmf_Dsigma2_interp + (sigma_eps2/sigma_cmf)*cdf_sig(om_cmf, sigma_cmf)).*prob_of_rectangle)...
             - DUMMYC*Dec_Dsigma2_interp.*frac_mon_ec_interp.*dens(ec,sigma_eps1);

phimc_sigma3 = sum((1-undef_cmf).*res2.*res3.*(dens(om_cmf_interp,sigma_eps1).*Domcmf_Dsigma3_interp + (sigma_eps3/sigma_cmf)*cdf_sig(om_cmf_interp, sigma_cmf)).*prob_of_rectangle)...
             - DUMMYC*Dec_Dsigma3_interp.*frac_mon_ec_interp.*dens(ec,sigma_eps1);


%%% Computing derivatives of shares (which are computed in choice.m)
sa_q = DUMMYB*Deb_Dq_interp*dens(eb,sigma_eps1);
sa_R = DUMMYB*Deb_DR_interp*dens(eb,sigma_eps1);
sa_tau = DUMMYB*Deb_Dtau_interp*dens(eb,sigma_eps1);
sa_varsigma = DUMMYB*Deb_Dvarsigma_interp*dens(eb,sigma_eps1);
sa_sigma1 = sum((1-undef_cmf).*(1-res3).*(1-undef_bank).*(1-res).*dens_sig1.*prob_of_rectangle);
sa_sigma2 = DUMMYB*Deb_Dsigma2_interp*dens(eb,sigma_eps1);
sa_sigma3 = DUMMYB*Deb_Dsigma3_interp*dens(eb,sigma_eps1);

sc_q   = - DUMMYC*Dec_Dq_interp*dens(ec,sigma_eps1);
sc_R   = - DUMMYC*Dec_DR_interp*dens(ec,sigma_eps1);
sc_tau = - DUMMYC*Dec_Dtau_interp*dens(ec,sigma_eps1);
sc_varsigma   = - DUMMYC*Dec_Dvarsigma_interp*dens(ec,sigma_eps1);
sc_sigma1 = sum((1-undef_cmf).*res2.*res3.*dens_sig1.*prob_of_rectangle);                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      
sc_sigma2 = - DUMMYC*Dec_Dsigma2_interp*dens(ec,sigma_eps1);
sc_sigma3 = - DUMMYC*Dec_Dsigma3_interp*dens(ec,sigma_eps1);

sb_q = - (sa_q + sc_q);
sb_R = - (sa_R + sc_R);
sb_tau = - (sa_tau + sc_tau);
sb_sigma1 = - (sa_sigma1 + sc_sigma1);
sb_sigma2 = - (sa_sigma2 + sc_sigma2);
sb_sigma3 = - (sa_sigma3 + sc_sigma3);
sb_varsigma = DUMMYC*Dec_Dvarsigma_interp*dens(ec,sigma_eps1) - DUMMYB*Deb_Dvarsigma_interp*dens(eb,sigma_eps1);
frac_bank_dropout_ec=normcdf( log( ed_ec + sigma_eps2^2/2)/sigma_eps2 );
frac_bank_continue_ec = 1-frac_bank_dropout_ec;
frac_bank_dropout_eb=normcdf( log( ed_eb + sigma_eps2^2/2)/sigma_eps2 );
frac_bank_continue_eb = 1-frac_bank_dropout_eb;
sbp_q = sb_q - DUMMYC*Dec_Dq_interp*dens(ec,sigma_eps1)*(1-frac_bank_continue_ec)...
      + DUMMYB*Deb_Dq_interp*dens(eb,sigma_eps1)*(1-frac_bank_continue_eb)...
      - sum((1-undef_bank).*res.*(1-res2).*DUMMYD.*Ded_Dq_interp.*dens(ed_eps1_interp,sigma_eps2).*prob_of_rectangle);
sbp_R = sb_R - DUMMYC*Dec_DR_interp*dens(ec,sigma_eps1)*(1-frac_bank_continue_ec)...
      + DUMMYB*Deb_DR_interp*dens(eb,sigma_eps1)*(1-frac_bank_continue_eb)...
      - sum((1-undef_bank).*res.*(1-res2).*DUMMYD.*Ded_DR_interp.*dens(ed_eps1_interp,sigma_eps2).*prob_of_rectangle);
sbp_tau = sb_tau - DUMMYC*Dec_Dtau_interp*dens(ec,sigma_eps1)*(1-frac_bank_continue_ec)...
        + DUMMYB*Deb_Dtau_interp*dens(eb,sigma_eps1)*(1-frac_bank_continue_eb);
frac_bank_continue_interp = interp1([s_bot-1,sgrd_bank,s_top+1],[0,frac_bank_continue,0],s_ng);
sbp_sigma1 = sb_sigma1 - sum((1-undef_bank).*res.*(1-res2).*(1-frac_bank_continue_interp).*dens_sig1.*prob_of_rectangle);
    sbp_sigma2_bit1 = sum((res_o.*(ones(le,1)*(dens_sig2.*prob_of_rect_eps2)))');
    sbp_sigma2_bit1_interp = interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,sbp_sigma2_bit1,0],s_ng);
sbp_sigma2 = sb_sigma2 - DUMMYC*Dec_Dsigma2_interp*dens(ec,sigma_eps1)*(1-frac_bank_continue_ec)...
           + DUMMYB*Deb_Dsigma2_interp*dens(eb,sigma_eps1)*(1-frac_bank_continue_eb)...
           + sum((1-undef_bank).*res.*(1-res2).*(sbp_sigma2_bit1_interp).*prob_of_rectangle);
sbp_sigma3 = sb_sigma3 - DUMMYC*Dec_Dsigma3_interp*dens(ec,sigma_eps1)*(1-frac_bank_continue_ec)...
      + DUMMYB*Deb_Dsigma3_interp*dens(eb,sigma_eps1)*(1-frac_bank_continue_eb)...
      - sum((1-undef_bank).*res.*(1-res2).*DUMMYD.*Ded_Dsigma3_interp.*dens(ed_eps1_interp,sigma_eps2).*prob_of_rectangle);
sbp_varsigma = DUMMYC*Dec_Dvarsigma_interp*dens(ec,sigma_eps1)*frac_bank_continue_ec...
      - DUMMYB*Deb_Dvarsigma_interp*dens(eb,sigma_eps1)*frac_bank_continue_eb...
      - sum((1-undef_bank).*res.*(1-res2).*DUMMYD.*Ded_Dvarsigma_interp.*dens(ed_eps1_interp,sigma_eps2).*prob_of_rectangle);


%%% Computing phif_q and phif_R 
phif_q =   sum((1-undef_bank).*res.*(1-res2).*Fb_2_interp.*prob_of_rectangle)... 
         + sum((1-undef_cmf).*res2.*res3.*Fc_2_interp.*prob_of_rectangle);
phif_R =   sum((1-undef_bank).*res.*(1-res2).*Fb_3_interp.*prob_of_rectangle)... 
         + sum((1-undef_cmf).*res2.*res3.*Fc_3_interp.*prob_of_rectangle);
phif_tau = sum((1-undef_bank).*res.*(1-res2).*Fb_4_interp.*prob_of_rectangle); 
phif_varsigma =   sum((1-undef_bank).*res.*(1-res2).*Fb_5_interp.*prob_of_rectangle);
phif_sigma1 = sa_sigma1 + sum((1-undef_bank).*res.*(1-res2).*Fb_interp.*dens_sig1.*prob_of_rectangle)...
                        + sum((1-undef_cmf).*res2.*res3.*Fc_interp.*dens_sig1.*prob_of_rectangle);
phif_sigma2 = sum((1-undef_bank).*res.*(1-res2).*Fb_sigma2_interp.*prob_of_rectangle)... 
         + sum((1-undef_cmf).*res2.*res3.*Fc_sigma2_interp.*prob_of_rectangle);;
phif_sigma3 = sum((1-undef_bank).*res.*(1-res2).*Fb_sigma3_interp.*prob_of_rectangle)... 
         + sum((1-undef_cmf).*res2.*res3.*Fc_sigma3_interp.*prob_of_rectangle);
     
%%% Computing derivatives of phirb (entering the equation for the risk premium on loans)
phirb =  sum( (1-undef_bank).*res.*(1-res2).*rp_bank.*prob_of_rectangle );
rp_bank_ec = rp_bank(j_cmf);
rp_bank_eb = rp_bank(j_bank);
rp_bank_q_bit1 = (1/Rnom)*(xi/(xi-1))*sum((res_o.*(s_times_eps1_grd'*eps2_grd).*((1/s_aggr)*om+Dombank_Dq).*(ones(le,1)*eps2_weight))'); 
r1_ed = ((1/Rnom)*(xi/((xi-1)))*TFP*(s_times_eps1_grd.*eps2_cutoff).*om_ed-1);
r1_ed_interp = interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,r1_ed,0],s_ng);
rp_bank_q_interp = - DUMMYD.*Ded_Dq_interp.*r1_ed_interp.*dens(ed_eps1_interp,sigma_eps2)...
            + interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,rp_bank_q_bit1 ,0],s_ng);
phirb_q = DUMMYC*Dec_Dq_interp*rp_bank_ec*dens(ec,sigma_eps1)...
        - DUMMYB*Deb_Dq_interp*rp_bank_eb*dens(eb,sigma_eps1)...
        + sum((1-undef_bank).*res.*(1-res2).*rp_bank_q_interp.*prob_of_rectangle);

rp_bank_R_bit1 = (1/Rnom)*(xi/(xi-1))*sum((res_o.*(s_times_eps1_grd'*eps2_grd).*(Dombank_DR-(1/Rnom)*om).*(ones(le,1)*eps2_weight))'); 
rp_bank_R_interp = - DUMMYD.*Ded_DR_interp.*r1_ed_interp.*dens(ed_eps1_interp,sigma_eps2)...
            + interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,rp_bank_R_bit1 ,0],s_ng);
phirb_R = DUMMYC*Dec_DR_interp*rp_bank_ec*dens(ec,sigma_eps1)...
        - DUMMYB*Deb_DR_interp*rp_bank_eb*dens(eb,sigma_eps1)...
        + sum((1-undef_bank).*res.*(1-res2).*rp_bank_R_interp.*prob_of_rectangle);
    
phirb_tau = DUMMYC*Dec_Dtau_interp*rp_bank_ec*dens(ec,sigma_eps1)...
          - DUMMYB*Deb_Dtau_interp*rp_bank_eb*dens(eb,sigma_eps1);

rp_bank_varsigma_bit1 = (1/Rnom)*(xi/(xi-1))*sum((res_o.*(s_times_eps1_grd'*eps2_grd).*Dombank_Dvarsigma.*(ones(le,1)*eps2_weight))'); 
rp_bank_varsigma_interp = - DUMMYD.*Ded_Dvarsigma_interp.*r1_ed_interp.*dens(ed_eps1_interp,sigma_eps2)...
            + interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,rp_bank_varsigma_bit1 ,0],s_ng);
phirb_varsigma = DUMMYC*Dec_Dvarsigma_interp*rp_bank_ec*dens(ec,sigma_eps1)...
        - DUMMYB*Deb_Dvarsigma_interp*rp_bank_eb*dens(eb,sigma_eps1)...
        + sum((1-undef_bank).*res.*(1-res2).*rp_bank_varsigma_interp.*prob_of_rectangle);

phirb_sigma1 = sum( (1-undef_bank).*res.*(1-res2).*rp_bank.*dens_sig1.*prob_of_rectangle );

rp_bank_sigma2_bit1 = sum((res_o.*((1/Rnom)*(xi/(xi-1))*(s_times_eps1_grd'*eps2_grd).*om-1).*(ones(le,1)*(dens_sig2.*prob_of_rect_eps2)))'); 
rp_bank_sigma2_interp = interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,rp_bank_sigma2_bit1 ,0],s_ng);
phirb_sigma2 = DUMMYC*Dec_Dsigma2_interp*rp_bank_ec*dens(ec,sigma_eps1)...
              - DUMMYB*Deb_Dsigma2_interp*rp_bank_eb*dens(eb,sigma_eps1)...
              + sum((1-undef_bank).*res.*(1-res2).*rp_bank_sigma2_interp.*prob_of_rectangle);

rp_bank_sigma3_bit1 = (1/Rnom)*(xi/(xi-1))*sum((res_o.*(s_times_eps1_grd'*eps2_grd).*Dombank_Dsigma3.*(ones(le,1)*eps2_weight))'); 
rp_bank_sigma3_interp = - DUMMYD.*Ded_Dsigma3_interp.*r1_ed_interp.*dens(ed_eps1_interp,sigma_eps2)...
                        + interp1([s_bot-1,s_times_eps1_grd,s_top+1],[0,rp_bank_sigma3_bit1 ,0],s_ng);
phirb_sigma3 = DUMMYC*Dec_Dsigma3_interp*rp_bank_ec*dens(ec,sigma_eps1)...
              - DUMMYB*Deb_Dsigma3_interp*rp_bank_eb*dens(eb,sigma_eps1)...
              + sum((1-undef_bank).*res.*(1-res2).*rp_bank_sigma3_interp.*prob_of_rectangle);
    
%%% Computing derivatives of phirc (entering the equation for the risk premium on bonds)
om_cmf_interp = interp1([s_bot-1,sgrd_cmf,s_top+1],[0,om_cmf,0],s_ng);
rp1_cmf = (1/Rnom)*(xi/(xi-1))*TFP*om_cmf_interp.*s_ng-1;
phirc =  sum( (1-undef_cmf).*res2.*res3.*rp1_cmf.*prob_of_rectangle );

rp1_cmf_ec = rp1_cmf(j_cmf);
phirc_q_bit1_interp = (1/Rnom)*(xi/(xi-1))*s_ng.*((1/s_aggr)*om_cmf_interp+Domcmf_Dq_interp);
phirc_q = - DUMMYC*Dec_Dq_interp*rp1_cmf_ec*dens(ec,sigma_eps1)...
          + sum((1-undef_cmf).*res2.*res3.*phirc_q_bit1_interp.*prob_of_rectangle);

phirc_R = - DUMMYC*Dec_DR_interp*rp1_cmf_ec*dens(ec,sigma_eps1)...
          + sum((1-undef_cmf).*res2.*res3.*(1/Rnom)*(xi/(xi-1)).*s_ng.*(Domcmf_DR_interp-(1/Rnom)*om_cmf_interp).*prob_of_rectangle);

phirc_tau = - DUMMYC*Dec_Dtau_interp*rp1_cmf_ec*dens(ec,sigma_eps1);

phirc_varsigma = - DUMMYC*Dec_Dvarsigma_interp*rp1_cmf_ec*dens(ec,sigma_eps1);

phirc_sigma1 =  sum((1-undef_cmf).*res2.*res3.*rp1_cmf.*dens_sig1.*prob_of_rectangle);

phirc_sigma2 = - DUMMYC*Dec_Dsigma2_interp*rp1_cmf_ec*dens(ec,sigma_eps1)...
          + sum((1-undef_cmf).*res2.*res3.*((1/Rnom)*(xi/(xi-1))*s_ng.*Domcmf_Dsigma2_interp).*prob_of_rectangle);

phirc_sigma3 = - DUMMYC*Dec_Dsigma3_interp*rp1_cmf_ec*dens(ec,sigma_eps1)...
          + sum((1-undef_cmf).*res2.*res3.*((1/Rnom)*(xi/(xi-1))*s_ng.*Domcmf_Dsigma3_interp).*prob_of_rectangle);

toc;
