% January 2015, F. DE FIORE and H. UHLIG. "Corporate debt structure and the financial crisis", JMCB 2015 and ECB WP No. 1759.
% dynamic_risk_ea_BOTH computes the impulse responses to aggregate shocks plotted in figures 4-6 of the paper


if ~exist('DOING_SEARCH'),
    DOING_SEARCH = 0;
end;

if ~DOING_SEARCH,
    disp('IRF in the DeFiore-Uhlig model ');
    disp('Version: January 2015');
end;

% Calculating the steady state and setting other parameters:
if ~exist('DO_STEADY_CALCULATIONS'),
DO_STEADY_CALCULATIONS = 1;  
end;
if ~exist('DO_PERMANENT_SHOCK');
   DO_PERMANENT_SHOCK=0;
end;
if (~DO_PERMANENT_SHOCK); 
    DO_STEADY_CALCULATIONS = 1;
end;

%The commands below are used to compute the impulse responses to transitory shocks under the various scenarios considered in section 5 of the paper,  
%i.e. i) full flexibility; ii) no bond issuance; and iii) no bond issuance and no bank flexibility

%%%To replicate figure 4 in the paper:  
% sets in this file DO_NO_CMF_IN_CHOICE = 0 ; DUMMYB = DUMMYC = DUMMYD = 1; 
% select the tau shock on line 491 of this file
% select the size of the shock (2.5) in line 75 of impresp_cs

%%% To replicate figure 5 in the paper:  
% Solid line: set DO_NO_CMF_IN_CHOICE = 0 ; DUMMYB = DUMMYC = DUMMYD = 1; %
% Dotted line: set DO_NO_CMF_IN_CHOICE = 1 ; DUMMYB = DUMMYC = DUMMYD = 1; %

%%% To replicate figure 6 in the paper: 
% Solid line: set DO_NO_CMF_IN_CHOICE = 0 ; DUMMYB = DUMMYC = DUMMYD = 1; %
% Dashed line: set DO_NO_CMF_IN_CHOICE = 1 ; DUMMYB = 1; DUMMYC = DUMMYD = 0; %

if DO_STEADY_CALCULATIONS,   
    DO_NO_CMF_IN_CHOICE = 0 ; % if 1, only the very largest epsilon-1 entrepreneur gets CMF, thus effectively shutting down CMF funding; this option is set in choice.m
    steadystate_search;
end; 
  
% Compute the bits necessary to solve for the thresholds   
if ~exist('DO_COEFFICIENTS'), 
DO_COEFFICIENTS = 1; 
end;
 
 
tic   
if DO_COEFFICIENTS,
   DUMMYB = 1; % = 0, if endogeneity of epsilon-b is to be turned off, = 1 otherw
   DUMMYC = 1; % = 0, if endogeneity of epsilon-c is to be turned off, = 1 otherw
   DUMMYD = 1; % = 0, if endogeneity of epsilon-d is to be turned off, = 1 otherw
   coefficients_risk_BOTH;
end;


    %Set other coefficients
    rho = .95; %.99
    rho_a = .95; %.99
    rho_ef = .95;%.95; %.12
    rho_tau = .95; %.99
    rho_sigma1 = .95; %.99
    rho_sigma2 = .95; %.99
    rho_sigma3 = .95; %.99
    rk=r;
    zeta=1; %1.5;
    pibar = pi_bar;
    nu = pibar;
     
    sigma_es = 0.24;  
    sigma_ef = 0.24;
    sigma_a = 0.02;
    sigma_tau = .02; %.02
    sigma_varkappa = .01;
    sigma_varsigma = .01;
    sigma_sigma1 = .01;
    sigma_sigma2 = .01;
    sigma_sigma3 = .01;
   

DO_LOAD_STEADY = 0;
DO_SAVE_STEADY = 0;
 
if DO_SAVE_STEADY,
        save steadyresults_ea
    end;

if DO_LOAD_STEADY,
    load steadyresults_ea;
end;


% Endogenous state variables:
%    1   2   3   4    5      
%    k   z   d   ms   R      
% Endogenous other variables:
% Endogenous other variables:
%    1   2   3   4   5   6   7  8     9    10   11   12    13    14   15   16  17   18   19  20  21  22  23  24  25   26  27    28  29   
%    y   e   n   q   pi  ya  x  lb   dty   rpb  rpc  defb  defc  def  sa   sb  sbp  sc   m   tr  h   w   rk  c   soa  I   rpd   yp  I_y  
% Exogenous variables:
%    1   2    3     4     5          6         7           8             9
%    a   es   tau   ef    varkappa   varsigma  sigma_eps1  sigma_eps2    sigma_eps3



 VARNAMES = [...
    '    H capital           '%1
    '    E capital           '%2
    '    deposits            '%3
    '    money supply        '%4
    '    risk-free rate      '%5
    '    output              '%6
    '    E cons              '%7 
    '    net worth           '%8
    '    markup              '%9
    '    inflation           '%10
    '    agency costs        '%11
    '    total funds         '%12
    '    loans to bonds      '%13
    '    debt to GDP         '%14
    '    spread loans        '%15
    '    spread bonds        '%16
    '    def loans           '%17
    '    def bonds           '%18
    '    aver default        '%19
    '    share abst          '%20
    '    share bank          '%21
    '    share bank/produce  '%22
    '    share CMF           '%23
    '    money               '%24
    '    transfers           '%25
    '    labor               '%26
    '    wage                '%27
    '    capital return      '%28
    '    H cons              '%29
    '     share overall abst '%30
    '   investment           '%31
    '   rp bonds-rp loans    '%32
    '   gdp                  '%33
    '     I to gdp           '%34
    'a:  tfp                 '%35
    'es: surprise ms         '%36
    'tau: bank cost          '%37
    'ef: anticipated ms      '%38
    'varkappa: net worth     '%39
    'varsigma: bank financing'%40
    'sigma1: risk sh 1       '%41
    'sigma2: risk sh 2       '%42
    'sigma3: risk sh 3       '];%43


% Switch notation.  Find matrices for format
% 0 = AA x(t) + BB x(t-1) + CC y(t) + DD z(t)
% 0 = E_t [ FF x(t+1) + GG x(t) + HH x(t-1) + JJ y(t+1) + KK y(t) + LL z(t+1) + MM z(t)]
% z(t+1) = NN z(t) + epsilon(t+1) with E_t [ epsilon(t+1) ] = 0,

det_eqs = 32;
exp_eqs = 2;
m_states = 5;
n_endog  = 29;
k_exog   = 9;

AA = zeros(det_eqs,m_states);
BB = zeros(det_eqs,m_states);
CC = zeros(det_eqs,n_endog);
DD = zeros(det_eqs,k_exog);
FF = zeros(exp_eqs,m_states);
GG = zeros(exp_eqs,m_states);
HH = zeros(exp_eqs,m_states);
JJ = zeros(exp_eqs,n_endog);
KK = zeros(exp_eqs,n_endog); 
LL = zeros(exp_eqs,k_exog);
MM = zeros(exp_eqs,k_exog);


%CONTEMPORANEOUS EQUATIONS
% 0 = AA x(t) + BB x(t-1) + CC y(t) + DD z(t)

%Equation 1: 0 = z(t) - (phif_R*Rnom/phif)*R(t) - n(t) - (phif_q*q/phif)*q(t) - (phif_tau*tau/phif)*tau(t) - varkappa(t) 
%                - (phif_sigma1*sigma_eps1/phif)*sigma_eps1(t) - (phif_sigma2*sigma_eps2/phif)*sigma_eps2(t) - (phif_sigma3*sigma_eps3/phif)*sigma_eps3(t)
AA(1,2) = 1; AA(1,5) = - (phif_R*Rnom/phif);
CC(1,3) =  - 1; CC(1,4) = - (phif_q*s_aggr/phif); 
DD(1,3) = - (phif_tau*tau/phif);
DD(1,5) = - 1;
DD(1,6) = - (phif_varsigma/phif);
DD(1,7) = - phif_sigma1*sigma_eps1/phif;
DD(1,8) = - phif_sigma2*sigma_eps2/phif;
DD(1,9) = - phif_sigma3*sigma_eps3/phif;

% Equation 2: 0 = k*k(t) + z*z(t) - (1-delta)*k*k(t-1)- (1-delta)*z*z(t-1) - yy(t)  + e*e(t) + ya*ya(t)  + c*c(t) 
AA(2,1)= k; AA(2,2)= z; 
BB(2,1)= -(1-delta)*k; BB(2,2)= -(1-delta)*z;
CC(2,1)= -y;  CC(2,2) = e; CC(2,6) = ya; CC(2,24) = c;
  
%%% Equation 3:  0 =  - k*k(t) + (1-delta+rk)*k*k(t-1)  + m*m(t) + w*l*h(t) + w*l*w(t) + rk*k*rk(t) - c*c(t)
AA(3,1)= -k;
BB(3,1)= (1-delta+rk)*k;  
CC(3,19)= m; CC(3,21)= w*l; CC(3,22)= w*l; CC(3,23)= rk*k;  CC(3,24)= -c; 

% Equation 4:  0 = - d*d(t) + ms*ms(t) - m*m(t) 
AA(4,3)= -d; AA(4,4)= ms; 
CC(4,19)= -m; 

% Equation 5: 0 = z(t-1)- n(t) + beta*rk*rk(t)
BB(5,2)= 1;
CC(5,3)= -1; CC(5,23)= beta*rk;

%%% Equation 6: 0 = (((1-tau)*sbp_R+sc_R)/((1-tau)*sbp+sc))*Rnom*R(t) + n(t) + (((1-tau)*sbp_q+sc_q)/((1-tau)*sbp+sc))*s_aggr*q(t)  
%%%                 - x(t)  + ( ((1-tau)*sbp_tau + sc_tau - sbp) /((1-tau)*sbp + sc))*tau*tau(t) 
%%%                 + (((1-tau)*sbp_varsigma+sc_varsigma)/((1-tau)*sbp+sc))*varsigma(t)
%%%                 + (((1-tau)*sbp_sigma1+sc_sigma1)/((1-tau)*sbp+sc))*sigma_eps1*sigma_eps1(t)
%%%                 + (((1-tau)*sbp_sigma2+sc_sigma2)/((1-tau)*sbp+sc))*sigma_eps2*sigma_eps2(t)
%%%                 + (((1-tau)*sbp_sigma3+sc_sigma3)/((1-tau)*sbp+sc))*sigma_eps3*sigma_eps3(t)
AA(6,5)= (((1-tau)*sbp_R+sc_R)/((1-tau)*sbp+sc))*Rnom; 
CC(6,3)= 1; CC(6,4)= (((1-tau)*sbp_q+sc_q)/((1-tau)*sbp+sc))*s_aggr; CC(6,7) = -1; 
DD(6,3) = ( ((1-tau)*sbp_tau + sc_tau - sbp) / ((1-tau)*sbp + sc))*tau;
DD(6,6) = (((1-tau)*sbp_varsigma+sc_varsigma)/((1-tau)*sbp+sc));
DD(6,7) = (((1-tau)*sbp_sigma1+sc_sigma1)/((1-tau)*sbp+sc))*sigma_eps1;
DD(6,8) = (((1-tau)*sbp_sigma2+sc_sigma2)/((1-tau)*sbp+sc))*sigma_eps2;
DD(6,9) = (((1-tau)*sbp_sigma3+sc_sigma3)/((1-tau)*sbp+sc))*sigma_eps3;

%%% Equation 7: 0 = (phiy_R*Rnom/phiy)*R(t) - y(t) +  n(t)+ (1 + s_aggr*phiy_q/phiy)*q(t)  + (phiy_tau*tau/phiy)*tau(t)
%%%                 + (phiy_varsigma/phiy)*varsigma(t)
%%%                 + (phiy_sigma1*sigma_eps1/phiy)*sigma_eps1(t)
%%%                 + (phiy_sigma2*sigma_eps2/phiy)*sigma_eps2(t)
%%%                 + (phiy_sigma3*sigma_eps3/phiy)*sigma_eps3(t)
AA(7,5)= phiy_R*Rnom/phiy;
CC(7,1)= -1; CC(7,3)= 1; CC(7,4)= 1 + s_aggr*phiy_q/phiy; 
DD(7,3) = phiy_tau*tau/phiy;
DD(7,6) = phiy_varsigma/phiy;
DD(7,7) = phiy_sigma1*sigma_eps1/phiy;
DD(7,8) = phiy_sigma2*sigma_eps2/phiy;
DD(7,9) = phiy_sigma3*sigma_eps3/phiy;


%Equation 8: 0 = w(t) - (1/kappa)*h(t) - zeta*c(t)   
CC(8,22)= 1; CC(8,24) = -zeta; 
CC(8,21)= -1/kappa;

% Equation 9: 0 =  - q(t) - alpha*w(t) - (1-alpha)*rk(t) + a(t)
CC(9,4) = -1; CC(9,22) = -alpha; CC(9,23) = - (1-alpha);
DD(9,1) = 1;

% Equation 10: 0 = (k/K)*k(t-1) + (z/K)*z(t-1) - x(t) + rk(t)   
BB(10,1) = k/K; BB(10,2) = z/K;  
CC(10,7) = -1; CC(10,23) = 1;

% Equation 11: 0 =  - x(t) + h(t) + w(t)
CC(11,7) = -1; CC(11,21) = 1; CC(11,22) = 1; 

%%% Equation 12: 0 = -ms(t) + ms(t-1) - pi(t)  + (1/pibar)*es(t) + (1/pibar)*ef(t)
AA(12,4) = -1;
BB(12,4) = 1;
CC(12,5) = -1; 
DD(12,2) = 1/pibar; DD(12,4) =1/pibar;

%%% Equation 13: 0 =  -(nu-1)*ms(t-1) + (nu-1)*pi(t) +(pi_bar*y/ms)*tr(t) - es(t)  - ef(t)
BB(13,4) = -(nu-1); 
CC(13,5) = (nu-1);
CC(13,20) = (pi_bar*y/ms); 
DD(13,2) = -1; DD(13,4) = -1;


% Equation 14: 0 = - d(t) + x(t)
AA(14,3)= -1; 
CC(14,7) = 1;

%Equation 15:  0 = Rnom*n*((1-tau)*phimb_R*mon*xi+phimc_R*mon*xi+tau*sb_R)*R(t) + ya*n(t) 
%                   + s_aggr*n*((1-tau)*phimb_q*mon*xi+phimc_q*mon*xi+tau*sb_q)*q(t) - ya*ya(t) 
%                   + (sb - phimb*mon*xi + (1-tau)*phimb_tau*mon*xi + phimc_tau*mon*xi + tau*sb_tau)*tau*n*tau(t)
%                   + n*((1-tau)*phimb_varsigma*mon*xi+phimc_varsigma*mon*xi+tau*sb_varsigma)*varsigma(t)
%                   + ((1-tau)*phimb_sigma1+phimc_sigma1+tau*sb_sigma1)*sigma_eps1*sigma_eps1(t)
%                   + ((1-tau)*phimb_sigma2+phimc_sigma2+tau*sb_sigma2)*sigma_eps2*sigma_eps2(t)
%                   + ((1-tau)*phimb_sigma3+phimc_sigma3+tau*sb_sigma3)*sigma_eps3*sigma_eps3(t)
AA(15,5) = Rnom*n*((1-tau)*phimb_R*mon*xi+phimc_R*mon*xi+tau*sb_R);
CC(15,3) = ya; CC(15,4) = s_aggr*n*((1-tau)*phimb_q*mon*xi+phimc_q*mon*xi+tau*sb_q); CC(15,6) = -ya;
DD(15,3) = (sb - phimb*mon*xi + (1-tau)*phimb_tau*mon*xi + phimc_tau*mon*xi + tau*sb_tau)*tau*n;
DD(15,6) = n*((1-tau)*phimb_varsigma*mon*xi+phimc_varsigma*mon*xi+tau*sb_varsigma);
DD(15,7) = ((1-tau)*phimb_sigma1*mon*xi+phimc_sigma1*mon*xi+tau*sb_sigma1)*n*sigma_eps1;
DD(15,8) = ((1-tau)*phimb_sigma2*mon*xi+phimc_sigma2*mon*xi+tau*sb_sigma2)*n*sigma_eps2;
DD(15,9) = ((1-tau)*phimb_sigma3*mon*xi+phimc_sigma3*mon*xi+tau*sb_sigma3)*n*sigma_eps3;

%Equation 16: 0 = e(t) - *z(t) 
AA(16,2) = -1; 
CC(16,2) = 1; 

% Equation 17: 0 =  ((sbp_R*sc-sbp*sc_R)/(sbp*sc))*Rnom*R(t) + ((sbp_q*sc-sbp*sc_q)/(sbp*sc))*s_aggr*q(t) - lb(t) 
%                + ((sbp_tau*sc-sbp*sc_tau-sbp*sc/(1-tau))/(sbp*sc))*tau*tau(t) +  ((sbp_varsigma*sc-sbp*sc_varsigma)/(sbp*sc))*varsigma(t)
%                   + ((sbp_sigma1*sc-sbp*sc_sigma1)*sigma_eps1/(sbp*sc))*sigma_eps1(t)
%                   + ((sbp_sigma2*sc-sbp*sc_sigma2)*sigma_eps2/(sbp*sc))*sigma_eps2(t)
%                   + ((sbp_sigma3*sc-sbp*sc_sigma3)*sigma_eps3/(sbp*sc))*sigma_eps3(t)
AA(17,5) = Rnom*((sbp_R*sc-sbp*sc_R)/(sbp*sc));
CC(17,4) = s_aggr*(sbp_q*sc-sbp*sc_q)/(sbp*sc); 
CC(17,8) = -1; 
DD(17,3) = tau*(sbp_tau*sc-sbp*sc_tau-sbp*sc/(1-tau))/(sbp*sc);
DD(17,6) = (sbp_varsigma*sc-sbp*sc_varsigma)/(sbp*sc);
DD(17,7) = (sbp_sigma1*sc-sbp*sc_sigma1)*sigma_eps1/(sbp*sc);
DD(17,8) = (sbp_sigma2*sc-sbp*sc_sigma2)*sigma_eps2/(sbp*sc);
DD(17,9) = (sbp_sigma3*sc-sbp*sc_sigma3)*sigma_eps3/(sbp*sc);

%Define debt to income ratio:
%Equation 18: 0 =  - dty(t)+ d(t) - y(t)
AA(18,3) = 1;
CC(18,1) = -1;
CC(18,9) = -1; 

% Equation 19: 0 = ((phirb_R*sbp-sbp_R*phirb)/(sbp^2))*Rnom*R(t) +
% ((phirb_q*sbp-sbp_q*phirb)/(sbp^2))*s_aggr*q(t) - risk_premium_bank*rpb(t)
%                   + ((phirb_tau*sbp-sbp_tau*phirb)/(sbp^2))*tau*tau(t) + ((phirb_varsigma*sbp-sbp_varsigma*phirb)/(sbp^2))* varsigma(t)
%                   + ((phirb_sigma1*sbp-sbp_sigma1*phirb)/(sbp^2))*sigma_eps1*sigma_eps1(t)
%                   + ((phirb_sigma2*sbp-sbp_sigma2*phirb)/(sbp^2))*sigma_eps2*sigma_eps2(t)
%                   + ((phirb_sigma3*sbp-sbp_sigma3*phirb)/(sbp^2))*sigma_eps3*sigma_eps3(t)
AA(19,5) = ((phirb_R*sbp-sbp_R*phirb)/(sbp^2))*Rnom;
CC(19,4) = ((phirb_q*sbp-sbp_q*phirb)/(sbp^2))*s_aggr; CC(19,10) = -risk_premium_bank; 
DD(19,3) = ((phirb_tau*sbp-sbp_tau*phirb)/(sbp^2))*tau;
DD(19,6) = ((phirb_varsigma*sbp-sbp_varsigma*phirb)/(sbp^2));
DD(19,7) = ((phirb_sigma1*sbp-sbp_sigma1*phirb)/(sbp^2))*sigma_eps1;
DD(19,8) = ((phirb_sigma2*sbp-sbp_sigma2*phirb)/(sbp^2))*sigma_eps2;
DD(19,9) = ((phirb_sigma3*sbp-sbp_sigma3*phirb)/(sbp^2))*sigma_eps3;

% Equation 20: 0 = ((phirc_R*sc-sc_R*phirc)/(sc^2))*Rnom*R(t) + ((phirc_q*sc-sc_q*phirc)/(sc^2))*s_aggr*q(t) - risk_premium_cmf*rpc(t)
%                  + ((phirc_tau*sc-sc_tau*phirc)/(sc^2))*tau*tau(t) + ((phirc_varsigma*sc-sc_varsigma*phirc)/(sc^2))*varsigma
%                   + ((phirc_sigma1*sc-sc_sigma1*phirc)/(sc^2))*sigma_eps1*sigma_eps1(t)
%                   + ((phirc_sigma2*sc-sc_sigma2*phirc)/(sc^2))*sigma_eps2*sigma_eps2(t)
%                   + ((phirc_sigma3*sc-sc_sigma3*phirc)/(sc^2))*sigma_eps3*sigma_eps3(t);
AA(20,5) = ((phirc_R*sc-sc_R*phirc)/(sc^2))*Rnom;
CC(20,4) = ((phirc_q*sc-sc_q*phirc)/(sc^2))*s_aggr; CC(20,11) = - risk_premium_cmf;
DD(20,3) = ((phirc_tau*sc-sc_tau*phirc)/(sc^2))*tau;
DD(20,6) = ((phirc_varsigma*sc-sc_varsigma*phirc)/(sc^2));
DD(20,7) = ((phirc_sigma1*sc-sc_sigma1*phirc)/(sc^2))*sigma_eps1;
DD(20,8) = ((phirc_sigma2*sc-sc_sigma2*phirc)/(sc^2))*sigma_eps2;
DD(20,9) = ((phirc_sigma3*sc-sc_sigma3*phirc)/(sc^2))*sigma_eps3;

% Equation 21: 0 = ((phimb_R*sbp-sbp_R*phimb)/(sbp^2))*Rnom*R(t) + ((phimb_q*sbp-sbp_q*phimb)/(sbp^2))*s_aggr*q(t) 
%                  - cond_monitored_if_continue_and_banking*defb(t) + ((phimb_tau*sbp-sbp_tau*phimb)/(sbp^2))*tau*tau(t)
%                  + ((phimb_varsigma*sbp-sbp_varsigma*phimb)/(sbp^2))*varsigma(t)
%                  + ((phimb_sigma1*sbp-sbp_sigma1*phimb)/(sbp^2))*sigma_eps1*sigma_eps1(t)
%                  + ((phimb_sigma2*sbp-sbp_sigma2*phimb)/(sbp^2))*sigma_eps2*sigma_eps2(t)
%                  + ((phimb_sigma3*sbp-sbp_sigma3*phimb)/(sbp^2))*sigma_eps3*sigma_eps3(t);
AA(21,5) = ((phimb_R*sbp-sbp_R*phimb)/(sbp^2))*Rnom;
CC(21,4) = ((phimb_q*sbp-sbp_q*phimb)/(sbp^2))*s_aggr; CC(21,12) = - cond_monitored_if_continue_and_banking; 
DD(21,3) = ((phimb_tau*sbp-sbp_tau*phimb)/(sbp^2))*tau;
DD(21,6) = ((phimb_varsigma*sbp-sbp_varsigma*phimb)/(sbp^2));
DD(21,7) = ((phimb_sigma1*sbp-sbp_sigma1*phimb)/(sbp^2))*sigma_eps1;
DD(21,8) = ((phimb_sigma2*sbp-sbp_sigma2*phimb)/(sbp^2))*sigma_eps2;
DD(21,9) = ((phimb_sigma3*sbp-sbp_sigma3*phimb)/(sbp^2))*sigma_eps3;

% Equation 22: 0 = ((phimc_R*sc-sc_R*phimc)/(sc^2))*Rnom*R(t) + ((phimc_q*sc-sc_q*phimc)/(sc^2))*s_aggr*q(t) - cond_monitored_if_cmf*defc(t)
%                  + ((phimc_tau*sc-sc_tau*phimc)/(sc^2))*tau*tau(t) + ((phimc_varsigma*sc-sc_varsigma*phimc)/(sc^2))*varsigma(t)
%                   + ((phimc_sigma1*sc-sc_sigma1*phimc)/(sc^2))*sigma_eps1*sigma_eps1(t)
%                   + ((phimc_sigma2*sc-sc_sigma2*phimc)/(sc^2))*sigma_eps2*sigma_eps2(t)
%                   + ((phimc_sigma3*sc-sc_sigma3*phimc)/(sc^2))*sigma_eps3*sigma_eps3(t);
AA(22,5) = ((phimc_R*sc-sc_R*phimc)/(sc^2))*Rnom;
CC(22,4) = ((phimc_q*sc-sc_q*phimc)/(sc^2))*s_aggr; CC(22,13) = - cond_monitored_if_cmf; 
DD(22,3) = ((phimc_tau*sc-sc_tau*phimc)/(sc^2))*tau;
DD(22,6) = ((phimc_varsigma*sc-sc_varsigma*phimc)/(sc^2));
DD(22,7) = ((phimc_sigma1*sc-sc_sigma1*phimc)/(sc^2))*sigma_eps1;
DD(22,8) = ((phimc_sigma2*sc-sc_sigma2*phimc)/(sc^2))*sigma_eps2;
DD(22,9) = ((phimc_sigma3*sc-sc_sigma3*phimc)/(sc^2))*sigma_eps3;

% Equations 23: 0 = (((phimb_R+phimc_R)*(sbp+sc)-(sbp_R+sc_R)*(phimb+phimc))/((sbp+sc)^2)))*Rnom*R(t) + (((phimb_q+phimc_q)*(sbp+sc)-(sbp_q+sc_q)*(phimb+phimc))/((sbp+sc)^2)))*s_aggr*q(t) 
%                  - aver_default*def(t) + (((phimb_tau+phimc_tau)*(sbp+sc)-(sbp_tau+sc_tau)*(phimb+phimc))/((sbp+sc)^2)))*tau*tau(t)
%                 + (((phimb_varsigma+phimc_varsigma)*(sbp+sc)-(sbp_varsigma+sc_varsigma)*(phimb+phimc))/((sbp+sc)^2)))*varsigma(t)
%                   + (((phimb_sigma1+phimc_sigma1)*(sbp+sc)-(sbp_sigma1+sc_sigma1)*(phimb+phimc))/((sb+sc)^2))*sigma_eps1*sigma_eps1(t)
%                   + (((phimb_sigma2+phimc_sigma2)*(sbp+sc)-(sbp_sigma2+sc_sigma2)*(phimb+phimc))/((sb+sc)^2))*sigma_eps2*sigma_eps2(t)
%                   + (((phimb_sigma3+phimc_sigma3)*(sbp+sc)-(sbp_sigma3+sc_sigma3)*(phimb+phimc))/((sb+sc)^2))*sigma_eps3*sigma_eps3(t);
AA(23,5) = (((phimb_R+phimc_R)*(sbp+sc)-(sbp_R+sc_R)*(phimb+phimc))/((sbp+sc)^2))*Rnom;
CC(23,4) = (((phimb_q+phimc_q)*(sbp+sc)-(sbp_q+sc_q)*(phimb+phimc))/((sbp+sc)^2))*s_aggr; CC(23,14) = - aver_default; 
DD(23,3) = (((phimb_tau+phimc_tau)*(sbp+sc)-(sbp_tau+sc_tau)*(phimb+phimc))/((sbp+sc)^2))*tau;
DD(23,6) = ((phimb_varsigma+phimc_varsigma)*(sbp+sc)-(sbp_varsigma+sc_varsigma)*(phimb+phimc))/((sbp+sc)^2);
DD(23,7) = (((phimb_sigma1+phimc_sigma1)*(sbp+sc)-(sbp_sigma1+sc_sigma1)*(phimb+phimc))/((sb+sc)^2))*sigma_eps1;
DD(23,8) = (((phimb_sigma2+phimc_sigma2)*(sbp+sc)-(sbp_sigma2+sc_sigma2)*(phimb+phimc))/((sb+sc)^2))*sigma_eps2;
DD(23,9) = (((phimb_sigma3+phimc_sigma3)*(sbp+sc)-(sbp_sigma3+sc_sigma3)*(phimb+phimc))/((sb+sc)^2))*sigma_eps3;

% Equation 24: 0 =  sa_R*Rnom*R(t) + sa_q*s_aggr*q(t) - sa*sa(t)+
% sa_tau*tau*tau(t) + sa_varsigma*varsigma(t) + sa_sigma1*sigma_eps1*sigma_eps1(t)+ sa_sigma2*sigma_eps2*sigma_eps2(t)+ sa_sigma3*sigma_eps3*sigma_eps3(t);
AA(24,5) = sa_R*Rnom;
CC(24,4) = sa_q*s_aggr; CC(24,15) = -sa;
DD(24,3) = sa_tau*tau;
DD(24,6) = sa_varsigma;
DD(24,7) = sa_sigma1*sigma_eps1;
DD(24,8) = sa_sigma2*sigma_eps2;
DD(24,9) = sa_sigma3*sigma_eps3;

% Equation 25: 0 =  sb_R*Rnom*R(t) + sb_q*s_aggr*q(t) -sb*sb(t) +
% sb_tau*tau*tau(t)+ sb_varsigma*varsigma(t)+ sb_sigma1*sigma_eps1*sigma_eps1(t)+ sb_sigma2*sigma_eps2*sigma_eps2(t)+ sb_sigma3*sigma_eps3*sigma_eps3(t);
AA(25,5) = sb_R*Rnom;
CC(25,4) = sb_q*s_aggr; CC(25,16) = -sb;
DD(25,3) = sb_tau*tau;
DD(25,6) = sb_varsigma;
DD(25,7) = sb_sigma1*sigma_eps1;
DD(25,8) = sb_sigma2*sigma_eps2;
DD(25,9) = sb_sigma3*sigma_eps3;

% Equation 26: 0 =  sbp_R*Rnom*R(t) + sbp_q*s_aggr*q(t) - sbp*sbp(t) +
% sbp_tau*tau*tau(t)+ sbp_varsigma*varsigma(t)+ sbp_sigma1*sigma_eps1*sigma_eps1(t)+ sbp_sigma2*sigma_eps2*sigma_eps2(t)+ sbp_sigma3*sigma_eps3*sigma_eps3(t);
AA(26,5) = sbp_R*Rnom;
CC(26,4) = sbp_q*s_aggr; CC(26,17) = -sbp;
DD(26,3) = sbp_tau*tau;
DD(26,6) = sbp_varsigma;
DD(26,7) = sbp_sigma1*sigma_eps1;
DD(26,8) = sbp_sigma2*sigma_eps2;
DD(26,9) = sbp_sigma3*sigma_eps3;

% Equation 27: 0 =  sc_R*Rnom*R(t) + sc_q*s_aggr*q(t) - sc*sc(t) +
% sc_tau*tau*tau(t) + sc_varsigma*varsigma(t)+ sc_sigma1*sigma_eps1*sigma_eps1(t)+ sc_sigma2*sigma_eps2*sigma_eps2(t)+ sc_sigma3*sigma_eps3*sigma_eps3(t);
AA(27,5) = sc_R*Rnom;
CC(27,4) = sc_q*s_aggr; CC(27,18) = -sc;
DD(27,3) = sc_tau*tau;  
DD(27,6) = sc_varsigma;
DD(27,7) = sc_sigma1*sigma_eps1;
DD(27,8) = sc_sigma2*sigma_eps2;
DD(27,9) = sc_sigma3*sigma_eps3;
 
% Equation 28: 0 =  - sa*sa(t) - sb*sb(t) + sbp*sbp(t) + soa*soa(t);
CC(28,15) = - sa; CC(28,16) = - sb; CC(28,17) = sbp; CC(28,25) = soa;
 
% Equation 29: 0 =  -I*I(t) + k*k(t) + z*z(t) - (1-delta)*k*k(t) - (1-delta)*z*z(t)
AA(29,1) = k;
AA(29,2) = z; 
BB(29,1) = -(1-delta)*k;
BB(29,2) = -(1-delta)*z; 
CC(29,26) = -I;

% Equation 30: 0 =  + risk_premium_bank*rpb(t) - risk_premium_cmf*rpc(t) + risk_premium_diff*rpd(t)
CC(30,10) = risk_premium_bank; CC(30,11) = - risk_premium_cmf; CC(30,27) = risk_premium_diff;

% Equation 31: 0 =  -y*y(t) + ya*ya(t) + yp*yp(t) 
CC(31,1) = - y; CC(31,6) = ya; CC(31,28) = yp;

% Equation 32: 0 =  -I_y(t) + I(t) - y(t) 
CC(32,1) = -1; CC(32,26) = 1; CC(32,29) = -1;


% EXPECTATIONAL EQUATIONS:
% 0 = E_t [ FF x(t+1) + GG x(t) + H x(t-1) + JJ y(t+1) + KK y(t) + LL z(t+1) + MM z(t)]
 
% Equation 1: 0 = E_t [ beta*rk*rk(t+1) - zeta* c(t+1)  + zeta*c(t)]
JJ(1,23) = beta*rk; JJ(1,24) = -zeta;  
KK(1,24) = zeta;

% Equation 2: 0 = E_t [ -zeta* c(t+1) + R(t ) - pi(t+1) + zeta*c(t) ]
GG(2,5) = 1;
JJ(2,5) = -1; JJ(2,24) = -zeta;
KK(2,24) = zeta;


% AUTOREGRESSIVE MATRIX FOR z(t)
% z(t+1) = NN z(t) + epsilon(t+1) with E_t [ epsilon(t+1) ] = 0,
% (1) log(a(t+1)) - log(a)) = rho*( log(a(t) - log(a) ) + eps_a(t+1)
% (2)  es(t+1) = eps_es(t+1): iid unexpected monetary policy shock 
% (3) log(tau(t+1)) - log(tau)) = rho*( log(tau(t) - log(tau) ) + eps_tau(t+1)
% (4) ef(t+1) = rho_ef* ef(t) + eps_ef(t+1)
% (5) log(varkappa(t+1)) - log(varkappa)) = rho*( log(varkappa(t) - log(varkappa) ) + eps_varkappa(t+1)
% (6) log(varkappa(t+1)) - log(varkappa)) = rho*( log(varkappa(t) -log(varkappa) ) + eps_varkappa(t+1)
% (7) log(sigma_eps1(t+1)) - log(sigma_eps1)) = rho*( log(sigma_eps1(t) - log(sigma_eps1) ) + eps_sigma_eps1(t+1)
% (8) log(sigma_eps2(t+1)) - log(sigma_eps2)) = rho*( log(sigma_eps2(t) -log(sigma_eps2) ) + eps_sigma_eps2(t+1)
% (9) log(sigma_eps3(t+1)) - log(sigma_eps3)) = rho*( log(sigma_eps3(t) -log(sigma_eps3) ) + eps_sigma_eps3(t+1)

NN = [rho_a  0    0        0      0       0     0            0           0
      0      0    0        0      0       0     0            0           0  
      0      0    rho_tau  0      0       0     0            0           0
      0      0    0        rho_ef 0       0     0            0           0 
      0      0    0        0      rho     0     0            0           0
      0      0    0        0      0       rho   0            0           0
      0      0    0        0      0       0     rho_sigma1   0           0   
      0      0    0        0      0       0     0            rho_sigma2  0
      0      0    0        0      0       0     0            0           rho_sigma3];
  
Sigma = [ sigma_a^2    0       0           0          0                 0                0                   0                  0
            0      sigma_es^2  0           0          0                 0                0                   0                  0
            0          0       0           sigma_ef^2 0                 0                0                   0                  0 
            0          0       sigma_tau^2 0          0                 0                0                   0                  0
            0          0       0           0          sigma_varkappa^2  0                0                   0                  0
            0          0       0           0          0                 sigma_varsigma^2 0                   0                  0
            0          0       0           0          0                 0                sigma_sigma1        0                  0 
            0          0       0           0          0                 0                0                   sigma_sigma2       0
            0          0       0           0          0                 0                0                   0                  sigma_sigma3];
  
% Setting the options:
DO_QZ = 0;
PERIOD     = 4;  % number of periods per year, i.e. 12 for monthly, 4 for quarterly
GNP_INDEX  = 6; % Index of output among the variables selected for HP filter
SIM_SUBPLOT = 1;
HP_SELECT  = 1:(m_states+n_endog+k_exog); % Selecting the variables for the HP Filter calcs.
DO_SHOCK_RESP = 1; % = 1 if impulse responses to shocks shall be calculated, =0 if permanent shocks shall be calculated.
SELECT_SHOCKS = [3 8 9]; %combined shocks, size given in impresp_cs.m (lines 72) 
COMBINED_SHOCKS = 1; % = 1 if impulse responses to the combined shocks selected in SELECT_SHOCKS should be calculated. 
                     %In this case, the file impresp_cs is called by do_it.m 
DO_STATE_RESP = 0; % = 1 if impulse responses to deviations of state variables should be computed or if response to permament shocks is computed, = 0 otherwise
INIT_DATE = 4;     % Number of periods prior to shock.  Default = 4. For INIT_DATE = 1, one sees the inital value for the state variable.
%                  % For INIT_DATE > 0, variables are initially zero, before responding.
if DO_STATE_RESP,
SELECT_STATES = 1 : m_states; % select the states to which impulse responses should be plotted.
end;
HORIZON    = 160; %80; for permanent shocks;
DO_PLOTS     = 1;  % If impulse response plots should be made, = 0, if not.   
IMP_SUBPLOT  = 1;  % Set =1, if impulse responses should be plotted in a subplot (all IRFs in the same graph)    
IMP_SUB_FONT = 6;  % Size of the title and labels in the subplots
IMP_JOINT = 0;  % Set =1, if all responses should be plotted in one graph, =0, if not 
IMP_SINGLE = 0;  % If  each response should be plotted in a separate graph, =0 if not.
IMP_SELECT =  [13 20 21 22 23 15 16 19 9 14 31 33]; 
%   DO_NO_ZERO_RESPONSE =  0; % Set = 1 to avoid plotting responses near zero
%   ZERO_RESPONSE_LEVEL =  0.0001; % Defining near zero to be the interval [-ZERO_RESPONSE_LEVEL,ZERO_RESPONSE_LEVEL]
%   DO_SCALE_AXIS = 0; % if the axis should be rescaled. 
%                      % Choices: according to a given scale
%                      %       or according to the max. response among AXIS_SELECT
%   DO_GIVEN_AXIS_SCALE = 0; % Scale according to a given axis scale, see next variable.
%                      % Only use = 1, if you are familiar with scaling axis in MATLAB.
%                      % see help axis for more information.
%   AXIS_SCALE = [0,HORIZON,-1,1];   % The argument for the axis(.) command, 
%                      % normally a vector of length 4 = [xmin,xmax,ymin,ymax].
AXIS_SELECT= 1:(m_states+n_endog+k_exog);
%                      %  for rescaling the axis automatically according to the
%                      %  max. response in AXIS_SELECT.
%                      % It is a good idea to make this at least as large as IMP_SELECT.
DO_ENLARGE = 1;    % =1, if you need large letter sizes for overheads, say.
%                      % If set to 1, letter size will be manipulated by ENLARGE.M
DO_THICK_LINES = 1;% If you set DO_ENLARGE = 1, then you will also get thick lines with DO_THICK_LINES = 1;
%   PRINT_FIG  = 0;    % Set = 1, if you want to sent figures directly to printer
%   DO_COLOR_PRINT = 0;% Set = 1 together with PRINT_FIG = 1, if you want to sent figures
%                      % directly to the currently installed windows color printing device
SAVE_FIG   = 1;    % Set = 1, if you want to save figures as encapsulated postscript files.
%                      % The graphic names are automatically chosen.  E.g. for impulse responses,
%                      % the file names are imprespN.eps, where N = 1,2,...  .  
%                      % Note: you need to set PRINT_FIG = 0 to be able to use SAVE_FIG.
COLOR_FIG  = 0;    % Set = 0, if you want to save as black-and-white .eps files (old default)
%                      % Set = 1, if you want to save as color .eps files (new default)
%                      % Note: you need to set SAVE_FIG = 1 or SAVE_FIG_JPG = 1 and PRINT_FIG = 0.
% SAVE_FIG_JPG   = 1;% Set = 1, if you want to save figures as color jpg-files.
%                      % The graphic names are automatically chosen.  E.g. for impulse responses,
%                      % the file names are imprespN.jpg, where N = 1,2,...  .  
%                      % Note: you need to set PRINT_FIG = 0 to be able to use SAVE_FIG_JPG.
%   FILENAMEPIECE = '';% To change the naming of the impulse response files from e.g.
%                      % impresp1.eps to impresp_new_1.eps,
%                      % assign FILENAMEPIECE='_new_'.
%   DO_FIXED_TEXT_POS  = 0; 
%                      % If text labels should be put at TXT_MARKER, set = 1.
%                      % Alternative: use point of maximal response between zero and MAX_TXT_MARKER.
%   TXT_MARKER  = 3;   % Where to put the labels on the graph, a number between 1 and HORIZON/2.


% Starting the calculations: 

if ~DOING_SEARCH,
    DISPLAY_IMMEDIATELY = 1;
    FILENAMEPIECE = [sprintf('%1.0f%1.0f%1.0f %1.0f kappa%4.1f',DUMMYB,DUMMYC,DUMMYD,DO_NO_CMF_IN_CHOICE,kappa)];
    
    do_it; 
    
    %create_graphic_files;
    
  
    % Starting the calculations:
    %saving the variables needed to plot IRFs in Resp_mat, VARNAMES, Time_axis, init_zero_response and SELECT_SHOCKS
    %save temp_shock_ea, Resp_mat, VARNAMES, Time_axis, init_zero_response, SELECT_SHOCKS
    %To plot the IRF of e.g. k to a TFP shock, use command
    %plot(Time_axis, [init_zero_response(1,:) Resp_mat(1,:)]); title(VARNAMES(1))
else 
    TROUBLE_IND = 0;
    
    [e_equ,m_states] = size(FF);
    [l_equ,m_states] = size(AA);
    [l_equ,n_endog ] = size(CC);
    [l_equ,k_exog  ] = size(DD);
    
    TROUBLE_IND = abs(l_equ + e_equ - m_states - n_endog );
    
    message = '                                                                       ';
    warnings = [];
    optionsTOOLKIT;
    NaNindic = max(max(isnan(AA))) + ...
        max(max(isnan(BB))) + ...
        max(max(isnan(CC))) + ...
        max(max(isnan(DD))) + ... 
        max(max(isnan(FF))) + ...
        max(max(isnan(GG))) + ...
        max(max(isnan(HH))) + ...
        max(max(isnan(JJ))) + ...
        max(max(isnan(KK))) + ...
        max(max(isnan(LL))) + ...
        max(max(isnan(MM))) + ...
        max(max(isnan(NN)));
    TROUBLE_IND = NaNindic; 
    
    
    if TROUBLE_IND == 0,
        solve;
        if exist('Xi_select'),
            len_Xi = length(Xi_sortabs);
            TROUBLE_IND = ( Xi_sortabs(min(len_Xi,max(Xi_select)+1)) < 1 - TOL ) + ...
                ( Xi_sortabs(max(Xi_select)) > 1 + TOL );
        else
            TROUBLE_IND = TROUBLE_IND + 1;
        end;
    end;
end;